#!/bin/bash
#
# Last update: 2024.12.12


# 輸入 -h 或 --help 會顯示手動安裝說明
if [ "$1" == "-h" ] || [ "$1" == "--help" ]; then

   clear
   echo "===================================================================================================="
   echo ""
   echo "手動安裝 CXS 請執行 /opt/cpinst/cpinst/install_script/cxs.sh"
   echo ""
   echo "手動安裝 CSF 請執行 /opt/cpinst/cpinst/install_script/csf.sh"
   echo ""
   echo "手動安裝 remoteip(Apache Module) 請執行 /opt/cpinst/cpinst/install_script/remoteip.sh"
   echo ""
   echo "手動安裝 CageFS(CloudLinux) 請執行 /opt/cpinst/cpinst/install_script/cagefs.sh"
   echo ""
   echo "手動安裝 PHPSelector(CloudLinux) 請執行 /opt/cpinst/cpinst/install_script/phpselector.sh"
   echo ""
   echo "手動安裝 Apache lsapi 請執行 /opt/cpinst/cpinst/install_script/lsapi.sh"
   echo ""
   echo "手動安裝 cPanel面板 翻譯 請執行 /opt/cpinst/cpinst/install_script/themes.sh"
   echo ""
   echo "===================================================================================================="
   exit

fi

# 如果系統磁區是使用 LVM，會直接離開 Script
LVM_check=$(lsblk -f | grep LVM)
if [ "${LVM_check}" ]; then
   echo "此主機磁區為 LVM，並非 Ext4，請重灌系統改用 Ext4"
   exit
fi

# 如果系統為 CentOS 6 或 CentOS 7 或 AlmaLinux 8，會直接離開 Script
OS_version=$(cat /etc/redhat-release | grep 'release 6\|release 7\|release 8' 2>/dev/null)
if [ "${OS_version}" ]; then
   echo "此主機系統為 CentOS 6 或 CentOS 7 或 AlmaLinux 8，無法正常使用此 Script 安裝 cPanel 環境。"
   exit
fi

# 如果系統已經安裝 cPanel，會直接離開 Script
if [ -e /usr/local/cpanel/cpanel ]; then
   echo "此主機已經安裝 cPanel，無法使用此 Script"
   exit
fi

# 紀錄開始安裝時間
echo "開始安裝 : `date`" > /root/install_date

# 系統是 CentOS，才會詢問是否要轉為 CloudLinuxOS (只會詢問一次)

CT=`grep CloudLinux /etc/redhat-release`

if [ ! -e /usr/local/src/no_install_cloudlinux ]; then

if [ ! "$CT" ]; then
  ans=''
  while [ "$ans"=="y" -o "$ans"=="n" ]; do
     clear
     echo "==========================================================="
     echo "  偵測到不是 CloudLinux OS，您是否要先轉為 CloudLinux OS ? "
     echo "  (轉換後會自動重開機，請重新執行此程序) $0                "
     echo "                                                           "
     echo "  CloudLinux 要先有授權才能安裝                            "
     echo "                                                           "
     echo "  (y) 轉換為 Cloudlinux OS (n) 不進行轉換                  "
     echo "==========================================================="
     read -e -p "您的選擇是 ? ==> (y/n): " -i "n" ans
     case $ans in
      y)
         dnf install wget -y
         wget http://repo.cloudlinux.com/cloudlinux/sources/cln/cldeploy
         sh cldeploy -i
         rm -f cldeploy
         shutdown -rf now
         exit
         break;;
      n)
         installcl="no"
         touch /usr/local/src/no_install_cloudlinux
         break;;
      *)
         echo ""
         echo "輸入選擇為不正確 : $ans，請參考以下說明，輸入正確值"
         echo ""
         echo "(y) 轉換為 CloudLinux OS (n) 不進行轉換"
         sleep 3
         continue;;
     esac
  done
fi

fi

###############################
### 安裝 cPanel 基礎環境系統 ###
###############################

# 將 "[root@cpXX ~]#" 改成 "root@cpXX [~]#" (只會設定一次)
# \u 使用者名稱 \h 主機名稱 \w 目前工作目錄
if [ ! -e /usr/local/src/set_ps1 ]; then
   PS1="\u@\h [\w]# "
   echo 'PS1="\u@\h [\w]# "' >> ~/.bashrc
   touch /usr/local/src/set_ps1
fi

# 設定 ls alias，讓 ll 指令可顯示隱藏檔案 (只會設定一次)
if [ ! -e /usr/local/src/set_ls_alias ]; then
   echo "LS_OPTIONS='--color=tty -F -a -b -T 0';" >> /etc/bashrc
   echo "export LS_OPTIONS;" >> /etc/bashrc
   alias ll='ls -l --color=tty -F -a -b -T 0'
   alias ls='/bin/ls $LS_OPTIONS'
   touch /usr/local/src/set_ls_alias
fi

# 設定時區區域為 Taipei (只會設定一次)
if [ ! -e /usr/local/src/set_timezone ]; then
   /usr/bin/timedatectl set-timezone Asia/Taipei
   touch /usr/local/src/set_timezone
fi

# 系統基本設定
if [ ! -e /usr/local/src/os_basic_config ]; then

   # 系統日誌保留改成 12 week
   sed -i 's/^rotate 4/rotate 12/g' /etc/logrotate.conf

   # 系統日誌啟用壓縮
   sed -i 's/^#compress/compress/g' /etc/logrotate.conf

   # 註冊 CloudLinux Server (如果系統是 AlmaLinux，會跳過註冊)
   if [ "$CT" ]; then
      /usr/sbin/clnreg_ks --force
      touch /usr/local/src/update_cloudlinux_license
   fi

   # 更新系統套件並設定 Cron
   /usr/bin/dnf upgrade -y
   /usr/bin/echo "0 6 * * * root /usr/bin/dnf upgrade -y > /dev/null 2>&1" >> /etc/crontab

   # 關閉 SElinux
   /usr/bin/sed -i 's/^SELINUX=.*/SELINUX=disabled/g' /etc/selinux/config
   /usr/sbin/setenforce 0

   # 優化 sysctl.conf 設定
   echo '' >> /etc/sysctl.conf
   /usr/bin/cat /opt/cpinst/cpinst/config/sysctl_conf >> /etc/sysctl.conf
   /usr/sbin/sysctl -p

   # fix CSF MESSENGER bug (AlmaLinux9預設為關閉)
   systemctl stop nftables
   systemctl disable nftables

   # 關閉 KDUMP
   systemctl stop kdump.service
   systemctl disable kdump.service

   touch /usr/local/src/os_basic_config
fi

# 安裝與設定 chrony 時間對時 (只會設定一次)
if [ ! -e /usr/local/src/set_chrony ]; then
   /usr/bin/dnf install chrony -y
   sed -i 's/pool 2.almalinux.pool.ntp.org iburst/#pool 2.almalinux.pool.ntp.org iburst/g' /etc/chrony.conf
   sed -i 's/pool 2.cloudlinux.pool.ntp.org iburst/#pool 2.cloudlinux.pool.ntp.org iburst/g' /etc/chrony.conf
   echo 'server 103.17.9.155 iburst' >> /etc/chrony.conf
   echo 'server 10.100.9.155 iburst' >> /etc/chrony.conf
   echo 'server 10.168.32.155 iburst' >> /etc/chrony.conf
   #echo 'server tock.stdtime.gov.tw iburst' >> /etc/chrony.conf
   systemctl start chronyd
   systemctl enable chronyd
   touch /usr/local/src/set_chrony
fi

# 設定 SSH 安全性 (只會設定一次)
if [ ! -e /usr/local/src/update_sshd_conf ]; then
   touch /etc/ssh/sshd_config.d/02-security.conf
   /usr/bin/echo "Ciphers aes128-ctr,aes192-ctr,aes256-ctr" >> /etc/ssh/sshd_config.d/02-security.conf
   /usr/bin/echo "MACs hmac-sha1" >> /etc/ssh/sshd_config.d/02-security.conf
   sed -i "s/#UseDNS no/UseDNS no/g" /etc/ssh/sshd_config
   /usr/bin/systemctl restart sshd.service
   touch /usr/local/src/update_sshd_conf
fi

# 關閉 firewalld、啟用 iptables (只會設定一次)
if [ ! -e /usr/local/src/disable_firewalld ]; then
   /usr/bin/systemctl disable firewalld.service
   /usr/bin/systemctl stop firewalld.service
   /usr/bin/dnf -y install iptables-services
   /usr/bin/touch /etc/sysconfig/iptables
   /usr/bin/touch /etc/sysconfig/iptables6
   /usr/bin/systemctl start iptables
   /usr/bin/systemctl start ip6tables
   /usr/bin/systemctl enable iptables
   /usr/bin/systemctl enable ip6tables
   iptables -F
   iptables -X
   service iptables save
   touch /usr/local/src/disable_firewalld
fi

# 關閉 Core Dump
cat /etc/security/limits.conf  |grep -v  . -n -m 1 |cut -d : -f1 |xargs -i  sed -i  '{}c \\*\thard\tcore\t0' /etc/security/limits.conf
echo 'kernel.core_pattern = /dev/null/' >> /etc/sysctl.conf
echo 'fs.suid_dumpable = 0' >> /etc/sysctl.conf
sysctl -p
echo 'ulimit -S -c 0 > /dev/null 2>&1' >> /etc/profile

# 安裝必要套件 (只會安裝一次)
if [ ! -e /usr/local/src/install_need_package ]; then
   /bin/dnf -y install epel-release
   /bin/dnf clean all
   /bin/dnf -y install openldap-clients nss-pam-ldapd perl nc vim-enhanced tar jq
   /bin/dnf -y install traceroute net-tools elinks tcpdump mutt bc pciutils wget

   # 預設 AlmaLinux 最小安裝可能不會安裝 rsyslog，手動安裝
   /bin/dnf install -y rsyslog
   systemctl start rsyslog
   systemctl enable rsyslog

   touch /usr/local/src/install_need_package
fi

############################
### 顯示安裝 cPanel 選單 ###
############################

# 選擇主機使用者類型
ans=''
host_use_user=''
while [ "$ans"=="1" -o "$ans"=="2" ]; do
   clear
   echo "==================================================="
   echo "  請選擇 主機使用者 類型                           "
   echo "                                                   "
   echo "  (1) 遠振虛擬主機專用 ( 客戶主機請勿選取 )         "
   echo "  (2) 客戶主機專用 ( 單次部屬 & 管理服務 )          "
   echo "==================================================="
   read -e -p "您的選擇是 ? ==> (1/2): " -i "" ans
   case $ans in
        1)
           host_use_user="1"
           break;;
        2)
           host_use_user="2"
           break;;
        *)
           echo ""
           echo "輸入選擇為不正確 : $ans，請參考以下說明，輸入正確值"
           echo ""
           echo "(1) 自用虛擬主機 (2) 客戶主機"
           sleep 3
           continue;;
   esac
done

sleep 1
clear


# 定義各項目預設參數

# 1.外網網卡名稱
CPANEL_INTERFACE=`ip -o link show | awk -F': ' '{print $2}'|grep -v lo|head -n 1`

# 2.Main IP
MAIN_IP=`ip route get 1.2.3.4 | grep via | awk '{print $7}'`

# 3.Main Share IP
MAIN_SHARE_IP="請手動輸入正確資訊"
if (("${host_use_user}" == "2")); then
   MAIN_SHARE_IP="$MAIN_IP"
fi

# 4.Hostname
SERVER_HOSTNAME=`hostname`

# 5.第一組 DNS 主機名稱
NS0="請手動輸入正確資訊"
if (("${host_use_user}" == "2")); then
   NS0=""
fi

# 6.第二組 DNS 主機名稱
NS1="請手動輸入正確資訊"
if (("${host_use_user}" == "2")); then
   NS1=""
fi

# 7.接收 System 通知郵件
CUSTOMER_EMAIL="cplog@ipmi.tw"
if (("${host_use_user}" == "2")); then
   CUSTOMER_EMAIL="請手動輸入正確資訊"
fi

# 8.接收 cPanel 通知郵件
WHM_EMAIL="whmlog@ipmi.tw"
if (("${host_use_user}" == "2")); then
   WHM_EMAIL="請手動輸入正確資訊"
fi

# 9.PHP版本(套用的 EA4 Json 設定檔)
eapver_version="83"

# 10.安裝 Default.htm 帳號預設頁面
installindex="yes"
if (("${host_use_user}" == "2")); then
   installindex="no"
fi

# 11.安裝本公司台灣或美國方案
installpackages="請手動輸入正確資訊"
if (("${host_use_user}" == "2")); then
   installpackages="no"
fi

# 12.安裝 CXS
installcxs="yes"
if (("${host_use_user}" == "2")); then
   installcxs="no"
fi

# 13.安裝 CageFS、PHP Selector (CloudLinux)
installcagefs="yes"
if (("${host_use_user}" == "2")); then
   installcagefs="no"
fi

# 14.安裝 Installatron
installinstallatron="yes"
if (("${host_use_user}" == "2")); then
   installinstallatron="no"
fi

# 15.安裝 KernelCare
kernelupdaten="yes"
if (("${host_use_user}" == "2")); then
   kernelupdaten="no"
fi

# 16.安裝 Zabbix
zabbix="yes"
if (("${host_use_user}" == "2")); then
   zabbix="no"
fi

# 17.每小時寄信限制
mail_qty="300"

# 18.資料庫類型
mysql_version="8.0"

# 19.安裝 lsapi
lsapi="yes"

# 20.安裝 MySQL Governor (CloudLinux)
# install_dbgovernor="yes"
# if (("${host_use_user}" == "2")); then
#    install_dbgovernor="no"
# fi


# 依照選擇主機使用者類型，顯示不同確定清單

ans=''
while [ "$ans"=="y" -o "$ans"=="q" ]; do
   clear
   echo "========================================================================"
   echo "  以下為預設選項設定，請務必確認無誤，再進行下一步                      "
   echo "                                                                        "

if (("${host_use_user}" == "1")) || (("${host_use_user}" == "2")); then
   echo "1.外網網卡名稱 : ${CPANEL_INTERFACE}"
fi

if (("${host_use_user}" == "1")) || (("${host_use_user}" == "2")); then
   echo "2.Main IP : ${MAIN_IP}"
fi

if (("${host_use_user}" == "1")) || (("${host_use_user}" == "2")); then
   echo "3.Main Share IP : ${MAIN_SHARE_IP}"
fi

if (("${host_use_user}" == "1")) || (("${host_use_user}" == "2")); then
   echo "4.Hostname : ${SERVER_HOSTNAME}"
fi

if (("${host_use_user}" == "1")); then
   echo "5.第一組 DNS 主機名稱 : ${NS0}"
fi

if (("${host_use_user}" == "1")); then
   echo "6.第二組 DNS 主機名稱 : ${NS1}"
fi

if (("${host_use_user}" == "2")); then
   echo "7.接收 System 通知郵件(管理員信箱) : ${CUSTOMER_EMAIL}"
fi

if (("${host_use_user}" == "2")); then
   echo "8.接收 cPanel 通知郵件(管理員信箱) : ${WHM_EMAIL}"
fi

if (("${host_use_user}" == "1")) || (("${host_use_user}" == "2")); then
   echo "9.PHP版本 : ${eapver_version}"
fi

if (("${host_use_user}" == "1")); then
   echo "10.安裝 Default.htm 帳號預設頁面 : ${installindex}"
fi

if (("${host_use_user}" == "1")); then
   echo "11.安裝本公司台灣或美國方案 : ${installpackages}"
fi

if (("${host_use_user}" == "1")); then
   echo "12.安裝 CXS : ${installcxs}"
fi

if (("${host_use_user}" == "1")) || (("${host_use_user}" == "2")); then
   echo "13.安裝 CageFS、PHP Selector (CloudLinux) : ${installcagefs}"
fi

if (("${host_use_user}" == "1")); then
   echo "14.安裝 Installatron : ${installinstallatron}"
fi

if (("${host_use_user}" == "1")) || (("${host_use_user}" == "2")); then
   echo "15.安裝 KernelCare (單次部屬無須安裝) : ${kernelupdaten}"
fi

if (("${host_use_user}" == "1")); then
   echo "16.安裝 Zabbix : ${zabbix}"
fi

if (("${host_use_user}" == "2")); then
   echo "17.每小時寄信限制 : ${mail_qty}"
fi

if (("${host_use_user}" == "1")) || (("${host_use_user}" == "2")); then
   echo "18.資料庫類型 : ${mysql_version}"
fi

if (("${host_use_user}" == "1")) || (("${host_use_user}" == "2")); then
   echo "19.安裝 lsapi : ${lsapi}"
fi

# if (("${host_use_user}" == "1")) || (("${host_use_user}" == "2")); then
#    echo "20.安裝 MySQL Governor (CloudLinux) -- 最後安裝 : ${install_dbgovernor}"
# fi

   echo "                                                                        "
   echo "  輸入項目編號，即可修改選項設定                                        "
   echo "                                                                        "
   echo "  (y) 開始安裝 cPanel (n) 取消安裝 cPanel                               "
   echo "                                                                        "
   echo "========================================================================"
   read -e -p "您的選擇是 ?  " -i "" ans
   case $ans in
        1)
           clear
           echo ""
           echo "### 請改成正確的外網網卡名稱 ###"
           echo ""
           read -e -p "1.外網網卡名稱 : " -i "$CPANEL_INTERFACE" CPANEL_INTERFACE
           continue;;
        2)
           clear
           echo ""
           read -e -p "2.Main IP : " -i "$MAIN_IP" MAIN_IP
           continue;;
        3)
           clear
           echo ""
           echo "### 管理服務主機，請設定客戶指定的 Main Share IP ###"
           echo ""
           read -e -p "3.Main Share IP : " -i "" MAIN_SHARE_IP
           continue;;
        4)
           clear
           echo ""
           echo "虛擬主機 cpX.g-dns.com(台灣)、半專屬主機 hdX.g-dns.com(台灣)"
           echo "經銷主機 reX.cx901.com(台灣) 或 urX.cx901.com(美國)"
           echo "WP主機 wpX.g-dns.com(台灣) 或 uwpX.g-dns.com(美國)"
           echo "貼圖主機 phX.g-dns.com(台灣)、"
           echo "新加坡主機 sgX.g-dns.com、香港主機 hkX.g-dns.com"
           echo "日本主機 jpX.g-dns.com"
           echo ""
           echo "### 管理服務主機，請依照管理服務安裝需求表設定 ###"
           echo ""
           read -e -p "4.Hostname : " -i "$SERVER_HOSTNAME" SERVER_HOSTNAME
           continue;;
        5)
           clear
           echo ""
           echo "虛擬主機 nsX.g-dns.com、經銷主機 nsX.cx901.com"
           echo ""
           read -e -p "5.第一組 DNS 主機名稱 : " -i "" NS0
           continue;;
        6)
           clear
           echo ""
           echo "虛擬主機 nsX.g-dns.com、經銷主機 nsX.cx901.com"
           echo ""
           read -e -p "6.第二組 DNS 主機名稱 : " -i "" NS1
           continue;;
        7)
           clear
           echo ""
           echo "### 請設定 Linux 需求表 管理員信箱 ###"
           echo ""
           read -e -p "7.接收 System 通知郵件(管理員信箱) : " -i "$CUSTOMER_EMAIL" CUSTOMER_EMAIL
           continue;;
        8)
           clear
           echo ""
           echo "### 請設定 Linux 需求表 管理員信箱 ###"
           echo ""
           read -e -p "8.接收 cPanel 通知郵件(管理員信箱) : " -i "$WHM_EMAIL" WHM_EMAIL
           continue;;
        9)
           clear
           echo ""
           echo "### 請依照管理服務安裝需求表設定 ###"
           echo ""
           echo "Apache 2.4 / PHP 8.0 / cgi = 80"
           echo "Apache 2.4 / PHP 8.1 / cgi = 81"
           echo "Apache 2.4 / PHP 8.2 / cgi = 82"
	        echo "Apache 2.4 / PHP 8.3 / cgi = 83"
	        echo "Apache 2.4 / PHP 8.4 / cgi = 84"
           echo ""
           echo "例如 : PHP 8.4，就是輸入 84"
           echo ""
           read -e -p "9.PHP版本 : " -i "$eapver_version" eapver_version
           continue;;
        10)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           read -e -p "10.安裝 Default.htm 帳號預設頁面 : " -i "$installindex" installindex
           continue;;
        11)
           clear
           echo ""
           echo "台灣 = tw、美國 = usa、不安裝 = no"
           echo "亞洲虛擬主機，請直接複製目前現有主機的方案設定檔"
           echo ""
           read -e -p "11.安裝本公司台灣或美國方案 : " -i "" installpackages
           continue;;
        12)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           read -e -p "12.安裝 CXS : " -i "$installcxs" installcxs
           continue;;
        13)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           read -e -p "13.安裝 CageFS、PHP Selector (CloudLinux) : " -i "$installcagefs" installcagefs
           continue;;
        14)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           read -e -p "14.安裝 Installatron : " -i "$installinstallatron" installinstallatron
           continue;;
        15)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           echo "### Standard 管理服務客戶才需要安裝 ###"
           echo ""
           read -e -p "15.安裝 KernelCare : " -i "$kernelupdaten" kernelupdaten
           continue;;
        16)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           echo "### 單次部屬不安裝，Standard 管理服務要安裝 ###"
           echo ""
           read -e -p "16.安裝 Zabbix : " -i "$zabbix" zabbix
           continue;;
        17)
           clear
           echo ""
           echo "### 管理服務主機，請依照管理服務安裝需求表設定 ###"
           echo ""
           read -e -p "17.每小時寄信限制 : " -i "$mail_qty" mail_qty
           continue;;
        18)
           clear
           echo ""
           echo "### MySQL 請輸入 8.0 ###"
           echo "### MariaDB 請輸入 10.5 or 10.6 or 10.11 or 11.4 ###"
           echo ""
           read -e -p "18.資料庫類型 : " -i "$mysql_version" mysql_version
           continue;;
        19)
           clear
           echo ""
           echo "安裝 = yes、不安裝 = no"
           echo ""
           echo "自用虛擬主機 : 目前統一使用 lsapi php執行模式"
           echo "客戶主機 : 目前cPanel與Cloudlinux都有提供lsapi套件，目前統一使用 lsapi php執行模式"
           echo ""
           read -e -p "19.安裝 lsapi : " -i "$lsapi" lsapi
           continue;;
      #   20)
      #      clear
      #      echo ""
      #      echo "安裝 = yes、不安裝 = no"
      #      echo ""
      #      read -e -p "20.安裝 MySQL Governor (CloudLinux) -- 最後安裝 : " -i "$install_dbgovernor" install_dbgovernor
      #      continue;;
        y)
           echo ""
           echo "開始安裝 cPanel"

### 判斷輸入值是否正確，不正確不給安裝 ###

if [ "${MAIN_SHARE_IP}" = "請手動輸入正確資訊" ]; then
   echo ""
   echo "3.Main Share IP : 請手動輸入正確資訊"
   sleep 3
   continue
fi

if [ "${NS0}" = "請手動輸入正確資訊" ]; then
   echo ""
   echo "5.第一組 DNS 主機名稱 : 請手動輸入正確資訊"
   sleep 3
   continue
fi

if [ "${NS1}" = "請手動輸入正確資訊" ]; then
   echo ""
   echo "6.第二組 DNS 主機名稱 : 請手動輸入正確資訊"
   sleep 3
   continue
fi

if [ "${CUSTOMER_EMAIL}" = "請手動輸入正確資訊" ]; then
   echo ""
   echo "7.接收 cPanel 日誌郵件的電子郵件 : 請手動輸入正確資訊"
   sleep 3
   continue
fi

if [ "${WHM_EMAIL}" = "請手動輸入正確資訊" ]; then
   echo ""
   echo "8.提供聯絡信息的電子郵件 : 請手動輸入正確資訊"
   sleep 3
   continue
fi

if [ "${eapver_version}" != '80' ] && [ "${eapver_version}" != '81' ] && [ "${eapver_version}" != '82' ] && [ "${eapver_version}" != '83' ] && [ "${eapver_version}" != '84' ]; then
   echo ""
   echo "9.目前輸入 PHP 版本為 ${eapver_version} 無法使用，請重新輸入正確值"
   sleep 3
   continue
fi

if [ "${installpackages}" != "tw" ] && [ "${installpackages}" != "usa" ] && [ "${installpackages}" != "no" ]; then
   echo ""
   echo "11.安裝本公司台灣或美國方案，請手動輸入正確資訊"
   sleep 3
   continue
fi

if [ "${mysql_version}" != "8.0" ] && [ "${mysql_version}" != "10.5" ] && [ "${mysql_version}" != "10.6" ] && [ "${mysql_version}" != "10.11" ] && [ "${mysql_version}" != "11.4" ]; then
   echo ""
   echo "18.目前輸入 mysql 版本 ${mysql_version} 無法使用，請重新輸入正確值"
   sleep 3
   continue
fi

if [ "${lsapi}" != "yes" ] && [ "${lsapi}" != "no" ]; then
   echo ""
   echo "19.是否安裝 lsapi 選項 ${lsapi} 無法使用，請重新輸入正確值"
   sleep 3
   continue
fi

# if [ "${install_dbgovernor}" != "yes" ] && [ "${install_dbgovernor}" != "no" ]; then
#    echo ""
#    echo "20.是否安裝 MySQL Governor 選項 ${install_dbgovernor} 無法使用，請重新輸入正確值"
#    sleep 3
#    continue
# fi

           break;;
        n)
           echo ""
           echo "取消安裝 cPanel !!!"
           echo ""
           exit;;
        *)
           echo ""
           echo "輸入選擇為不正確 : $ans，請參考以下說明，輸入正確值"
           echo ""
           echo "(y) 開始安裝 cPanel (n) 取消安裝 cPanel"
           echo ""
           echo "或者 輸入項目編號，即可修改選項設定"
           sleep 3
           continue;;
   esac
done

# 紀錄選項設定值
echo " " >> /root/install_date
echo "######################" >> /root/install_date
echo " " >> /root/install_date
echo "Use: cpinst9_new.sh" >> /root/install_date
echo " " >> /root/install_date
echo "0.主機使用者類型：${host_use_user}" >> /root/install_date
echo "1.外網網卡名稱 : ${CPANEL_INTERFACE}" >> /root/install_date
echo "2.Main IP : ${MAIN_IP}" >> /root/install_date
echo "3.Main Share IP : ${MAIN_SHARE_IP}" >> /root/install_date
echo "4.Hostname : ${SERVER_HOSTNAME}" >> /root/install_date
echo "5.第一組 DNS 主機名稱 : ${NS0}" >> /root/install_date
echo "6.第二組 DNS 主機名稱 : ${NS1}" >> /root/install_date
echo "7.接收 cPanel 日誌郵件的電子郵件 : ${CUSTOMER_EMAIL}" >> /root/install_date
echo "8.提供聯絡信息的電子郵件 : ${WHM_EMAIL}" >> /root/install_date
echo "9.PHP版本 : ${eapver_version}" >> /root/install_date
echo "10.安裝 Default.htm 帳號預設頁面 : ${installindex}" >> /root/install_date
echo "11.安裝本公司台灣或美國方案 : ${installpackages}" >> /root/install_date
echo "12.安裝 CXS : ${installcxs}" >> /root/install_date
echo "13.安裝 CageFS、PHP Selector (CloudLinux) : ${installcagefs}" >> /root/install_date
echo "14.安裝 Installatron : ${installinstallatron}" >> /root/install_date
echo "15.安裝 KernelCare : ${kernelupdaten}" >> /root/install_date
echo "16.安裝 Zabbix : ${zabbix}" >> /root/install_date
echo "17.每小時寄信限制 : ${mail_qty}" >> /root/install_date
echo "18.資料庫類型 : ${mysql_version}" >> /root/install_date
echo "19.安裝 lsapi : ${lsapi}" >> /root/install_date
# echo "20.安裝 MySQL Governor (CloudLinux) -- 最後安裝 : ${install_dbgovernor}" >> /root/install_date
echo "######################" >> /root/install_date
echo " " >> /root/install_date

#######################
### 開始安裝 cPanel ###
#######################

# 設定正確 hostname
/usr/bin/hostnamectl set-hostname $SERVER_HOSTNAME

# 再次向 CloudLinux Server 註冊 (如果系統是 CentOS，會跳過註冊)
if [ "$CT" ]; then
   /usr/sbin/clnreg_ks --force
fi

# 設定 cPanel 使用更新為 stable
echo "CPANEL=stable" > /etc/cpupdate.conf

# 複製 cpanel.config 範例設定檔
mkdir -p /root/cpanel_profile
cp /opt/cpinst/cpinst/config/cpanel.config /root/cpanel_profile/

# 客戶主機重啟Apache服務間隔改成預設值10秒
if [ "$host_use_user" == "2" ]; then
   sed -i "s/^min_time_between_apache_graceful_restarts=.*/min_time_between_apache_graceful_restarts=10/g" /root/cpanel_profile/cpanel.config
fi

# 設定每小時寄信最大限制數量 (客戶主機 才會進行設定，自用主機都是使用預設值 300)
if [ "$host_use_user" == "2" ]; then
   sed -i "s/^maxemailsperhour=.*/maxemailsperhour=${mail_qty}/g" /root/cpanel_profile/cpanel.config
fi

# 設定資料庫類型
sed -i "s/^mysql-version=.*/mysql-version=${mysql_version}/g" /root/cpanel_profile/cpanel.config

# 複製 EA4 Json 範例設定檔
eapver="cpanel_install_ea4_profile_php_${eapver_version}_9.json"
cp /opt/cpinst/cpinst/config/${eapver} /etc/cpanel_initial_install_ea4_profile.json

# 避免因為非使用cloudlinux系統，部分套件無法安裝，導致客製化json檔案套用失敗 Ex.2023/11/7
if [ -e /usr/local/src/no_install_cloudlinux ]; then
   sed -i '/ea-libcurl/d' /etc/cpanel_initial_install_ea4_profile.json
   sed -i '/ea-libcurl-devel/d' /etc/cpanel_initial_install_ea4_profile.json
   sed -i '/ea-apache24-mod_hostinglimits/d' /etc/cpanel_initial_install_ea4_profile.json
   sed -i '/ea-openssl/d' /etc/cpanel_initial_install_ea4_profile.json
   sed -i '/ea-openssl-devel/d' /etc/cpanel_initial_install_ea4_profile.json
fi

# 複製 wwwacct.conf 範例設定檔，置換正確的設定
cp /opt/cpinst/cpinst/config/wwwacct.conf /etc/wwwacct.conf
sed -i "s/QQHSTNME/${SERVER_HOSTNAME}/g" /etc/wwwacct.conf
sed -i "s/QQINTERFACE/${CPANEL_INTERFACE}/g" /etc/wwwacct.conf
sed -i "s/QQWHMMAIL/${WHM_EMAIL}/g" /etc/wwwacct.conf
sed -i "s/QQMAINSHAREIP/${MAIN_SHARE_IP}/g" /etc/wwwacct.conf
sed -i "s/QQNAM1/${NS0}/g" /etc/wwwacct.conf
sed -i "s/QQNAM2/${NS1}/g" /etc/wwwacct.conf

# 開始安裝 cPanel，安裝紀錄存放 /var/log/cpanel-install.log
cd /root && curl -o latest -L https://securedownloads.cpanel.net/latest && sh latest

# 更新 cPanel license
/usr/local/cpanel/cpkeyclt

# 安裝本公司台灣或美國方案
if [ "$installpackages" == "tw" ]; then
   /bin/cp -f /opt/cpinst/cpinst/config/packages/tw/* /var/cpanel/packages/
fi
if [ "$installpackages" == "usa" ]; then
   /bin/cp -f /opt/cpinst/cpinst/config/packages/usa/* /var/cpanel/packages/
fi

####################
### 系統其他設定 ###
####################

# 關閉不必要的服務
cat <<'EOF_DISABLE_SERVICES' > /root/cpanel-disable-services
#!/bin/bash
for service in \
   avahi-daemon \
   avahi-dnsconfd \
   bluetooth \
   capi \
   conman \
   cpus \
   cups \
   irda \
   lisa \
   netplugd \
   ifplugd \
   netfs \
   netplugd \
   nfs \
   nfslock \
   nfsd \
   ncsd \
   oddjobd \
   openvpn \
   pcmcia \
   pcscd \
   portsentry \
   postfix \
   rawdevices \
   rdisc \
   rpcgssd \
   rpcidmapd \
   saslauthd \
   tcsd \
   vsftpd \
   winbind \
   wpa_supplicant \
   xfs \
   mdmpd \
   mdmonitor \
   canna \
   FreeWnn \
   cups-config-daemon \
   iiim \
   mDNSResponder \
   nifd \
   anacron \
   gpm \
   hidd \
   sbadm \
   qpidd \
   ypbind ;

   do
       echo "Disabling service: ${service}"
       service ${service} stop &>/dev/null
       systemctl stop ${service} &>/dev/null
       systemctl disable ${service} &>/dev/null
done
EOF_DISABLE_SERVICES
chmod 700 /root/cpanel-disable-services
/root/cpanel-disable-services
rm -f /root/cpanel-disable-services

####################
### WHM 初始設定 ###
####################

# 安裝 CSF 套件
sh /opt/cpinst/cpinst/install_script/csf.sh ${host_use_user}
systemctl start csf
systemctl start lfd
systemctl enable csf
systemctl enable lfd

# fix CSF MESSENGER Bug
systemctl stop nftables
systemctl disable nftables
csf -ra

# register appconfig for csf.php & restoreacct.conf
# check command : whmapi1 get_appconfig_application_list
if [ "$host_use_user" == "1" ]; then
   /usr/local/cpanel/bin/register_appconfig /opt/cpinst/cpinst/config/unblockip.conf
   mkdir -p /usr/local/cpanel/whostmgr/docroot/cgi/unblockip/
   /bin/cp -f /opt/cpinst/moni_script/csf.php /usr/local/cpanel/whostmgr/docroot/cgi/unblockip/
fi

# Apache mod_userdir Tweak
sed -i "s/DefaultHost:-1/DefaultHost:/g" /var/cpanel/moddirdomains
/scripts/userdirctl on
/scripts/restartsrv_httpd --hard

# 關閉 Compiler Access
/scripts/compilers off

# 關閉 cPHulk Brute Force Protection
cphulk=$(whmapi1 configureservice service='cphulkd' enabled='0' monitored='0')

# 設定 Edit System Mail Preferences
echo "cpanel: ${CUSTOMER_EMAIL}" > /etc/localaliases
echo 'nobody: /dev/null' >> /etc/localaliases
echo "root: ${CUSTOMER_EMAIL}" >> /etc/localaliases

# 複製 Contact Manager 範例設定檔
mv /var/cpanel/icontact_event_importance.json /var/cpanel/icontact_event_importance.json_bak
/bin/cp -f /opt/cpinst/cpinst/config/icontact_event_importance.json /var/cpanel/icontact_event_importance.json
/bin/cp -f /opt/cpinst/cpinst/config/clevels.conf /var/cpanel/clevels.conf

# Service Manager
servicemanager=$(whmapi1 configureservice service='cpdavd' enabled='0' monitored='0')

# Apache Configuration >> Global Configuration & DirectoryIndex Priority
sed -i 's/"root_options" : "[^"]*"/"root_options" : "FollowSymLinks IncludesNOEXEC ExecCGI"/' /etc/cpanel/ea4/ea4.conf
sed -i 's/"maxclients" : 150/"maxclients" : 750/' /etc/cpanel/ea4/ea4.conf
sed -i 's/"serverlimit" : 256/"serverlimit" : 750/' /etc/cpanel/ea4/ea4.conf
if [ "$host_use_user" == "2" ]; then
   sed -i 's/"maxclients" : 750/"maxclients" : 256/' /etc/cpanel/ea4/ea4.conf
   sed -i 's/"serverlimit" : 750/"serverlimit" : 256/' /etc/cpanel/ea4/ea4.conf
fi
sed -i 's/directoryindex" : "index.php index.php8 index.php7 index.php5 index.perl index.pl index.plx index.ppl index.cgi index.jsp index.jp index.phtml index.shtml index.xhtml index.html index.htm index.js"/directoryindex" : "index.php index.php8 index.php7 index.php5 index.perl index.pl index.plx index.ppl index.cgi index.jsp index.jp index.phtml index.shtml index.xhtml index.html index.htm index.js Default.htm"/' /etc/cpanel/ea4/ea4.conf
/scripts/rebuildhttpdconf
/scripts/restartsrv_httpd

# Apache Configuration > Include Editor
dnf install ea-apache24-mod_reqtimeout -y
/usr/bin/cat <<EOF >> /usr/local/apache/conf/includes/pre_main_global.conf
<IfModule mod_reqtimeout.c>
  RequestReadTimeout header=10-20,MinRate=500 body=20-600,MinRate=500
</IfModule>
EOF
/usr/bin/cat <<EOF >> /etc/apache2/conf.d/includes/pre_virtualhost_global.conf
  SSLUseStapling off
EOF
/scripts/rebuildhttpdconf
/scripts/restartsrv_httpd

# 安裝 RemoteIP 套件
if [ "$host_use_user" == "1" ]; then
   /bin/bash /opt/cpinst/cpinst/install_script/remoteip.sh
fi
if [ "$host_use_user" == "2" ]; then
   /bin/bash /opt/cpinst/cpinst/install_script/remoteip.sh
   sed -i '/add_remoteip/d' /etc/crontab
   cp /opt/cpinst/cpinst/install_script/add_remoteip.sh /root/add_remoteip.sh
   /bin/bash /root/add_remoteip.sh
   echo "30 0 * * * root /bin/bash /root/add_remoteip.sh > /dev/null 2>&1" >> /etc/crontab
fi

# 新增 Apache 網頁保護 config
if [ "$host_use_user" == "1" ]; then
   touch /etc/apache2/conf.d/includes/website_abuse.conf
   sed -i '1s/^/Include "\/usr\/local\/apache\/conf\/includes\/website_abuse.conf"\n/' /etc/apache2/conf.d/includes/pre_virtualhost_global.conf
   cp -v /opt/cpinst/adwin_script/fast_suspend_website.py /root/process_fingerprint/
   chmod -v +x /root/process_fingerprint/fast_suspend_website.py
   /scripts/rebuildhttpdconf
   /scripts/restartsrv_httpd
fi

# 阻擋客戶網站被存取.git目錄與.git與.env檔案
cp /opt/cpinst/cpinst/config/deny_git.conf /etc/apache2/conf.d/
cp /opt/cpinst/cpinst/config/deny_env.conf /etc/apache2/conf.d/
/scripts/restartsrv_httpd

# Exim Configuration Manager
if [ "$host_use_user" == "1" ]; then
   mkdir -p /var/cpanel/config.backups/
   /bin/cp /opt/cpinst/cpinst/config/whm-config-backup-cpanel__smtp__exim-116.001000-1739177001.tar.gz /var/cpanel/config.backups/
   /usr/local/cpanel/bin/cpconftool --restore=/var/cpanel/config.backups/whm-config-backup-cpanel__smtp__exim-116.001000-1739177001.tar.gz --modules=cpanel::smtp::exim

   touch /etc/exim_drop_users
   mkdir /root/process_fingerprint/
   cp /opt/cpinst/red_python/local_exim_connection_analyzer.py /root/process_fingerprint/
   cp /opt/cpinst/red_python/foreign_exim_connection_analyzer.py /root/process_fingerprint/
   cp /opt/cpinst/red_python/webmail_exim_connection_analyzer.py /root/process_fingerprint/
   echo "* * * * * /root/process_fingerprint/local_exim_connection_analyzer.py -q -f" >> /var/spool/cron/root
   echo "*/2 * * * * /root/process_fingerprint/foreign_exim_connection_analyzer.py -q -f" >> /var/spool/cron/root
   echo "*/2 * * * * /root/process_fingerprint/webmail_exim_connection_analyzer.py -q -f" >> /var/spool/cron/root
   chmod 755 /root/process_fingerprint/*.py

   touch /etc/excludesenderdomains
fi
if [ "$host_use_user" == "2" ]; then
   mkdir -p /var/cpanel/config.backups/
   /bin/cp /opt/cpinst/cpinst/config/whm-config-backup-cpanel__smtp__exim-116.001000-1734056477.tar.gz /var/cpanel/config.backups/
   /usr/local/cpanel/bin/cpconftool --restore=/var/cpanel/config.backups/whm-config-backup-cpanel__smtp__exim-116.001000-1734056477.tar.gz --modules=cpanel::smtp::exim
fi
#echo '*: '`cat /var/cpanel/mainip` > /etc/mailips
#echo '*: '`hostname` > /etc/mailhelo

# MultiPHP INI Editor > ea-phpxx php.ini
# https://api.docs.cpanel.net/openapi/whm/operation/php_ini_set_directives/
multiphp=$(whmapi1 php_ini_set_directives directive-1='display_errors:Off' directive-2='disable_functions:symlink,show_source, system, exec, shell_exec, passthru, popen, proc_open, proc_close, proc_get_status, proc_nice, proc_terminate, escapeshellarg, escapeshellcmd, ini_alter, define_syslog_variables, posix_uname, posix_getpwuid, posix_getpwnam, apache_child_terminate, posix_kill, posix_mkfifo, posix_setpgid, posix_setsid, posix_setuid, proc_terminate, posix_uname, diskfreespace, leak, base64_encodem, eval, apache_child_terminate, apache_setenv' directive-3='date.timezone:Asia/Taipei' directive-4='memory_limit:128M' version="ea-php${eapver_version}")

# 關閉 PHP-FPM
phpfpm=$(whmapi1 php_set_default_accounts_to_fpm default_accounts_to_fpm='0')

# Background Process Killer
processtokill=$(whmapi1 configurebackgroundprocesskiller processes_to_kill='BitchX' processes_to_kill-0='bnc' processes_to_kill-1='eggdrop' processes_to_kill-2='generic-sniffers' processes_to_kill-3='guardservices' processes_to_kill-4='ircd' processes_to_kill-5='psyBNC' processes_to_kill-6='ptlink' processes_to_kill-7='services')

# 更新 /etc/my.cnf 設定檔
mv /etc/my.cnf /etc/my.cnf_bak
if [ "$mysql_version" == "8.0" ]; then
   cp /opt/cpinst/cpinst/config/my_8.txt /etc/my.cnf
fi
if [ "${mysql_version}" == "10.5" ] && [ "${mysql_version}" == "10.6" ] && [ "${mysql_version}" == "10.11" ] && [ "${mysql_version}" == "11.4" ]; then
   cp /opt/cpinst/cpinst/config/my.txt /etc/my.cnf
   sed -i '/optimizer_switch/d' /etc/my.cnf
fi
/scripts/restartsrv_mysql

# 安裝 Default.htm 帳號預設頁面
if [ "$installindex" == "yes" ]; then
   mkdir -p /root/cpanel3-skel/public_html
   cp /opt/cpinst/cpinst/config/Default.htm /root/cpanel3-skel/public_html
fi

# pure-ftpd
/scripts/setupftpserver --force pure-ftpd
rm -f /var/cpanel/conf/pureftpd/main.cache
mv /var/cpanel/conf/pureftpd/main /var/cpanel/conf/pureftpd/main.backup
/bin/cp -f /opt/cpinst/cpinst/config/pureftpd_main /var/cpanel/conf/pureftpd/main
touch /var/cpanel/conf/pureftpd/local
echo 'ForcePassiveIP: ~' >> /var/cpanel/conf/pureftpd/local
echo 'PassivePortRange: 30000 50000' >> /var/cpanel/conf/pureftpd/local
echo 'TLSCipherSuite: ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:DHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384:TLS_AES_256_GCM_SHA384:TLS_CHACHA20_POLY1305_SHA256:TLS_AES_128_GCM_SHA256' >> /var/cpanel/conf/pureftpd/local
sed -i "s/PassivePortRange 49152 65534/PassivePortRange 30000 50000/g" /etc/pure-ftpd.conf
/scripts/setupftpserver --force pure-ftpd
/scripts/restartsrv_ftpserver

# AutoSSL
autossl=$(whmapi1 --output=jsonpretty set_autossl_metadata  metadata_json='{"clobber_externally_signed":"0","notify_autossl_expiry":"0","notify_autossl_expiry_coverage":"0","notify_autossl_expiry_coverage_user":"0","notify_autossl_expiry_user":"0","notify_autossl_renewal":"0","notify_autossl_renewal_coverage":"0","notify_autossl_renewal_coverage_reduced":"0","notify_autossl_renewal_coverage_reduced_user":"0","notify_autossl_renewal_coverage_user":"0","notify_autossl_renewal_uncovered_domains":"0","notify_autossl_renewal_uncovered_domains_user":"0","notify_autossl_renewal_user":"0"}')

# Greylisting
greylisting=$(whmapi1 disable_cpgreylist)

# 啟用 Two-Factor Authentication
twofactor=$(whmapi1 twofactorauth_enable_policy)

# 幫 hostname 安裝 免費憑證
if [ "$host_use_user" == "1" ]; then
   /usr/local/cpanel/bin/checkallsslcerts
fi

# 將FTP紀錄拆分至 /var/log/ftp，保留 24 weekly
if [ "$host_use_user" == "1" ]; then
touch /var/log/ftp
chmod 640 /var/log/ftp
chown root:root /var/log/ftp

touch /etc/rsyslog.d/ftp.conf
cat << EOF >> /etc/rsyslog.d/ftp.conf
if \$programname == 'pure-ftpd' then /var/log/ftp
& stop
EOF

systemctl restart rsyslog

touch /etc/logrotate.d/pure-ftpd
cat << EOF >> /etc/logrotate.d/pure-ftpd
/var/log/ftp {
    weekly
    rotate 24
    compress
    missingok
    notifempty
    create 640 root root
    sharedscripts
    postrotate
        /bin/systemctl reload rsyslog > /dev/null 2>/dev/null || true
    endscript
}
EOF

logrotate -d /etc/logrotate.d/pure-ftpd

sed -i 's/^FTPD_LOG.*.=.*/FTPD_LOG = "\/var\/log\/ftp"/g' /etc/csf/csf.conf
csf -ra
fi

# 設定 Default Style
whmapi1 set_default type='default' name='basic'
mkdir -pv /root/cpanel3-skel/.cpanel/nvdata/
echo -n "1" > /root/cpanel3-skel/.cpanel/nvdata/xmainNewStyleBannerDismissed

################
### 套件安裝 ###
################

# 安裝 CageFS
if [ "$installcagefs" == "yes" ]; then
   dnf clean packages
   /bin/bash /opt/cpinst/cpinst/install_script/cagefs.sh
   touch /etc/container/cl-quotas.dat
fi

# 抓資源限制之設定檔
if [ "$host_use_user" == "1" ]; then
if [ -e /etc/container/ve.cfg ]; then
   /bin/mv -f /etc/container/ve.cfg /etc/container/ve.cfg_bak
   /bin/cp -f /opt/cpinst/cpinst/config/phpselector/ve_cfg /etc/container/ve.cfg
   lvectl set default --vmem=0
   lvectl apply all
fi
fi

# 抓inode限制之設定檔
if [ "$host_use_user" == "1" ]; then
if [ -e /etc/container/cl-quotas.dat ]; then
   /bin/mv -f /etc/container/cl-quotas.dat /etc/container/cl-quotas.dat_bak
   /bin/cp -f /opt/cpinst/cpinst/config/phpselector/cl-quotas.dat /etc/container/cl-quotas.dat
   /usr/bin/cl-quota -YC
fi
fi

# 安裝 PHP Selector
if [ "$installcagefs" == "yes" ]; then
   /bin/bash /opt/cpinst/cpinst/install_script/phpselector.sh
fi

# 安裝 Installatron
if [ "$installinstallatron" == "yes" ]; then
   rpm -Uvh http://dal01.installatron.com/installatron-plugin-cpanel-latest.noarch.rpm
fi

# 安裝 CXS
if [ "$installcxs" == "yes" ]; then
   /bin/bash /opt/cpinst/cpinst/install_script/cxs.sh
fi

# 安裝 KernelCare
if [ "$kernelupdaten" == "yes" ]; then
   curl -s https://repo.cloudlinux.com/kernelcare/kernelcare_install.sh | bash
if [ "$CT" ]; then
/usr/bin/cat <<EOF >> /etc/sysctl.conf
 ## protect against symlink attack
fs.enforce_symlinksifowner=1
EOF
# kcarectl --set-patch-type extra --update
/usr/sbin/sysctl -p
fi
   kcarectl --update
fi

# 安裝 RAID 程式 (有兩層檢查，確認主機使用RAID卡，才會安裝RAID程式)
dnf install -y pciutils
raidcard_check=$(lspci | grep -i raid 2>/dev/null)
if [ "${raidcard_check}" ]; then
   wget -O /usr/local/src/raid_monitor_9.sh https://cpanel.net.tw/tool/megacli/raid_monitor_9.sh
   sh /usr/local/src/raid_monitor_9.sh
fi

# 安裝 Safe-rm
if [ "$host_use_user" == "1" ]; then
   /opt/cpinst/cpinst/install_script/safe-rm.sh
fi

# 安裝 SNMP
cd /usr/local/src
wget -O snmp.sh https://cpanel.net.tw/tool/snmp/snmp.txt
sh /usr/local/src/snmp.sh

# 安裝 Memcached
dnf install -y memcached
systemctl start memcached
systemctl enable memcached
netstat -ntlp | grep memcached
dnf install ea-php${eapver_version}-php-memcache -y
dnf install ea-php${eapver_version}-php-memcached -y

# 安裝 Zabbix
if [ "$zabbix" == "yes" ]; then

   cd /root
   git clone git@git.twnoc.net:nicole/zabbix.git
   sh /root/zabbix/zabbix_install.sh $SERVER_HOSTNAME

if [ "$host_use_user" == "1" ]; then
   phpchk_exists=$(whmapi1 listaccts --output=json | jq -e '.data.acct[]? | select(.user == "phpchk") | .user' 2>/dev/null)
   if [ -z "$phpchk_exists" ]; then
       if [ -d "/home/phpchk" ]; then
           if [ "$(stat -c "%U.%G" /home/phpchk)" = "root.root" ]; then
	       rm -rf /home/phpchk
           fi
       fi

      # 避免建立的帳號無法正常切換php版本，因此再次執行重裝套件以及更新設定 2019-9-4 Jacky
      dnf reinstall cagefs lvemanager lve-stats lve-utils -y
      /bin/mv -f /etc/cl.selector/php.conf /etc/cl.selector/php.conf.backup
      cp -f /opt/cpinst/cpinst/config/php_conf /etc/cl.selector/php.conf
      sed -i 's/^expose_php.*.=.*/expose_php = Off/g' /opt/alt/php*/etc/php.ini
      sed -i 's/^log_errors.*.=.*/log_errors = Off/g' /opt/alt/php*/etc/php.ini
      sed -i 's/^enable_dl.*.=.*/enable_dl = Off/g' /opt/alt/php*/etc/php.ini
      sed -i 's/^disable_functions.*.=.*/disable_functions = symlink,show_source, system, exec, shell_exec, passthru, popen, proc_open, proc_close, proc_get_status, proc_nice, proc_terminate, escapeshellarg, escapeshellcmd, ini_alter, define_syslog_variables,posix_uname,posix_getpwuid,posix_getpwnam, apache_child_terminate,posix_kill,posix_mkfifo,posix_setpgid,posix_setsid,posix_setuid,proc_terminate, posix_uname, diskfreespace, leak,base64_encodem,eval, apache_child_terminate, apache_setenv/g' /opt/alt/php*/etc/php.ini
      sed -i 's/date\.timezone.*.=.*/date\.timezone="Asia\/Taipei"/g' /opt/alt/php*/etc/php.ini
      sed -i 's/^;memory_limit.*.=.*/memory_limit = 128M/g' /opt/alt/php*/etc/php.ini
      sed -i 's/^upload_max_filesize.*/upload_max_filesize = 32M/g' /opt/alt/php*/etc/php.ini
      cagefsctl --remount-all
      cagefsctl --force-update

      /usr/bin/hostnamectl set-hostname $SERVER_HOSTNAME
      /usr/local/cpanel/bin/set_hostname $SERVER_HOSTNAME
      phpchk_domain=`hostname -s`.phpchk.com

      whmapi1 createacct username=phpchk domain=${phpchk_domain}

   fi

# PHP 虛擬主機新增 PHP Selector 與 remoteip 監控

#新增 php_select_test.php 程式
touch /home/phpchk/public_html/php_select_test.php
chown phpchk.phpchk /home/phpchk/public_html/php_select_test.php
cat << EOF > /home/phpchk/public_html/php_select_test.php
<?
echo phpversion();
?>
EOF

#新增 ip.php 程式
touch /home/phpchk/public_html/ip.php
chown phpchk.phpchk /home/phpchk/public_html/ip.php
cat << EOF > /home/phpchk/public_html/ip.php
<?php
echo \$_SERVER['REMOTE_ADDR'] . " " ;
echo \$_SERVER['SERVER_ADDR'] ;
?>
EOF

#新增 remoteip.php 程式
touch /home/phpchk/public_html/remoteip.php
chown phpchk.phpchk /home/phpchk/public_html/remoteip.php
mainip=`cat /var/cpanel/mainip`
cat << EOF > /home/phpchk/public_html/remoteip.php
<?php
\$ch = curl_init();
curl_setopt(\$ch,CURLOPT_URL,"http://${phpchk_domain}/ip.php");
curl_setopt(\$ch, CURLOPT_INTERFACE, "$mainip"); //ip 請填入此主機 main ip
curl_setopt(\$ch,CURLOPT_HEADER,0);//是否輸出header
\$out = curl_exec(\$ch);
curl_close(\$ch);
?>
EOF

fi

fi

# 安裝 lsapi
if [ "$lsapi" == "yes" ]; then
   /opt/cpinst/cpinst/install_script/lsapi.sh
fi

################
### 排程設定 ###
################

# 設定 惡意工作排程監控
if [ "$host_use_user" == "1" ]; then
   mkdir /root/process_fingerprint/
   cp /opt/cpinst/red_python/cron_security_check.py /root/process_fingerprint/
   chmod 755 /root/process_fingerprint/cron_security_check.py
   echo "0 23 * * * /root/process_fingerprint/cron_security_check.py -q -c" >> /var/spool/cron/root
fi

# 監控主機 sshd Port 22 是否有被 try
if [ "$host_use_user" == "1" ]; then
   echo "0 * * * * root /bin/bash /opt/cpinst/jacky_script/check_sshd.sh" >> /etc/crontab
fi

# 監控主機 LOAD
if [ "$host_use_user" == "1" ]; then
   echo "*/5 * * * * root /bin/bash /opt/cpinst/jacky_script/check_load.sh" >> /etc/crontab
fi

# 每日16:00執行SQL自動備份
if [ "$host_use_user" == "1" ]; then
   mkdir /backup
   touch /backup/sqldump_error.log
   echo '0 16 * * * root script -c /opt/cpinst/jacky_script/daily_dump_all_sql.sh /backup/sqldump_error.log' >> /etc/crontab
fi

# 限制cPanel帳號無法存取內部網路
if [ "$host_use_user" == "1" ]; then
   echo '0 */6 * * * root /bin/bash /opt/cpinst/jacky_script/cpanel_account_intranet_reject.sh --run > /dev/null 2>&1' >> /etc/crontab
fi

# 產生網域列表放在phpchk內提供工程內部網頁存取
if [ "$host_use_user" == "1" ]; then
   echo '* * * * * root /opt/cpinst/steven_script/update_cpuser_domain_list.sh > /dev/null 2>&1' >> /etc/crontab
fi

# 將獨立IP網域列表、排外網域列表 定期更新至 /etc/excludesenderdomains
if [ "$host_use_user" == "1" ]; then
   echo '*/10 * * * * root /opt/cpinst/jacky_script/auto_update_excludesenderdomains.sh --run' >> /etc/crontab
fi

####################
### 其他手動設定 ###
####################

# 設定cPanel面板
if [ "$host_use_user" == "1" ]; then

   # Paper_lantern > 調整顯示功能
   if [ "$host_use_user" == "1" ]; then
      cd /var/cpanel/features/
      echo "" > default
      echo "" > photo
      echo "" > single
      echo "" > addon
      echo "" > single-hd
      echo "" > disabled
      cat /opt/cpinst/cpinst/config/feature/default > 'default'
      cat /opt/cpinst/cpinst/config/feature/photo > 'photo'
      cat /opt/cpinst/cpinst/config/feature/single > 'single'
      cat /opt/cpinst/cpinst/config/feature/addon > 'addon'
      cat /opt/cpinst/cpinst/config/feature/single-hd > 'single-hd'
      cat /opt/cpinst/cpinst/config/feature/disabled > 'disabled'

      touch /usr/local/cpanel/base/frontend/jupiter/dynamicui/dynamicui_hide_icon.conf
      echo "file=>remote_mysql,skipobj=>1" > \
      /usr/local/cpanel/base/frontend/jupiter/dynamicui/dynamicui_hide_icon.conf
   fi

   # cPanel面板更新翻譯
   /opt/cpinst/cpinst/install_script/themes.sh

   # 限制用戶切換面板
   echo "jupiter" > /var/cpanel/themes.conf

fi

################
### Fix Bug ###
################

# fix not open webmail for roundcube problem
/usr/local/cpanel/bin/update-roundcube-db

# Add python link
ln -s /usr/bin/python3.9 /usr/bin/python

# 修正v106語系檔案消失問題 (v118沒有此問題，先暫時註解)
#/opt/cpinst/jacky_script/import_cplocales.sh

# remove WordPress-Toolkit-Plugin
# https://support.cpanel.net/hc/en-us/articles/360058369013-How-can-I-remove-the-WordPress-Toolkit-Plugin-
rpm -e wp-toolkit-cpanel

# 再次設定正確 hostname，避免DNS未指向，被cPanel改成xxxx.cprapid.com名稱
/usr/bin/hostnamectl set-hostname $SERVER_HOSTNAME
/usr/local/cpanel/bin/set_hostname $SERVER_HOSTNAME
echo '*: '`hostname` > /etc/mailhelo
ln -s /var/lib/mysql/mysql.err /var/lib/mysql/`hostname`.err

# 如果是 CloudLinuxOS，將加裝以下套件 (目前 php81 版本之後 json 沒包含這兩個套件)
if [ "$CT" ]; then
  dnf install -y ea-libcurl ea-libcurl-devel
fi

# 修正ipv6優先權高於ipv4問題
touch /etc/gai.conf
cat << EOF >> /etc/gai.conf
label ::1/128 0
label ::/0 1
label 2002::/16 2
label ::/96 3
label ::ffff:0:0/96 4
precedence ::1/128 50
precedence ::/0 40
precedence 2002::/16 30
precedence ::/96 20
precedence ::ffff:0:0/96 100
EOF

# 再次執行安裝 PHP Selector & Lsapi (有時候 alt-php 套件會安裝失敗，作業流程結束前再安裝一次)
if [ "$installcagefs" == "yes" ]; then
   /bin/bash /opt/cpinst/cpinst/install_script/phpselector.sh
fi
if [ "$lsapi" == "yes" ]; then
   /opt/cpinst/cpinst/install_script/lsapi.sh
fi

# 關閉 SElinux (避免沒關到，再執行一次)
/usr/bin/sed -i 's/^SELINUX=.*/SELINUX=disabled/g' /etc/selinux/config
/usr/sbin/setenforce 0

# 手動安裝此套件，避免 /var/log/messages 等等log檔案不會歸檔與壓縮 (2024/8/23)
dnf install -y rsyslog-logrotate

# CP32 & CP33 轉移後新視野踩到缺少檔案的坑
rpm --import https://wp-toolkit.plesk.com/cPanel/wp-toolkit-cpanel.gpg
dnf install -y libc-client

# https://support.cpanel.net/hc/en-us/articles/4413998544407-Locales-are-missing-on-Almalinux-8
dnf install -y glibc-all-langpacks

# 避免 whmapi1 指令執行失效，如果前面執行輸出結果為空，表示沒正常執行，故再執行一次
if [ -z "$cphulk" ]; then
   sleep 3
   whmapi1 configureservice service='cphulkd' enabled='0' monitored='0'
   echo "cphulk: 第一次執行為空" >> /root/install_date
fi
if [ -z "$servicemanager" ]; then
   sleep 3
   whmapi1 configureservice service='cpdavd' enabled='0' monitored='0'
   echo "servicemanager: 第一次執行為空" >> /root/install_date
fi
if [ -z "$multiphp" ]; then
   sleep 3
   whmapi1 php_ini_set_directives directive-1='display_errors:Off' directive-2='disable_functions:symlink,show_source, system, exec, shell_exec, passthru, popen, proc_open, proc_close, proc_get_status, proc_nice, proc_terminate, escapeshellarg, escapeshellcmd, ini_alter, define_syslog_variables, posix_uname, posix_getpwuid, posix_getpwnam, apache_child_terminate, posix_kill, posix_mkfifo, posix_setpgid, posix_setsid, posix_setuid, proc_terminate, posix_uname, diskfreespace, leak, base64_encodem, eval, apache_child_terminate, apache_setenv' directive-3='date.timezone:Asia/Taipei' directive-4='memory_limit:128M' version="ea-php${eapver_version}"
   echo "multiphp: 第一次執行為空" >> /root/install_date
fi
if [ -z "$phpfpm" ]; then
   sleep 3
   whmapi1 php_set_default_accounts_to_fpm default_accounts_to_fpm='0'
   echo "phpfpm: 第一次執行為空" >> /root/install_date
fi
if [ -z "$processtokill" ]; then
   sleep 3
   whmapi1 configurebackgroundprocesskiller processes_to_kill='BitchX' processes_to_kill-0='bnc' processes_to_kill-1='eggdrop' processes_to_kill-2='generic-sniffers' processes_to_kill-3='guardservices' processes_to_kill-4='ircd' processes_to_kill-5='psyBNC' processes_to_kill-6='ptlink' processes_to_kill-7='services'
   echo "processtokill: 第一次執行為空" >> /root/install_date
fi
if [ -z "$autossl" ]; then
   sleep 3
   whmapi1 --output=jsonpretty set_autossl_metadata  metadata_json='{"clobber_externally_signed":"0","notify_autossl_expiry":"0","notify_autossl_expiry_coverage":"0","notify_autossl_expiry_coverage_user":"0","notify_autossl_expiry_user":"0","notify_autossl_renewal":"0","notify_autossl_renewal_coverage":"0","notify_autossl_renewal_coverage_reduced":"0","notify_autossl_renewal_coverage_reduced_user":"0","notify_autossl_renewal_coverage_user":"0","notify_autossl_renewal_uncovered_domains":"0","notify_autossl_renewal_uncovered_domains_user":"0","notify_autossl_renewal_user":"0"}'
   echo "autossl: 第一次執行為空" >> /root/install_date
fi
if [ -z "$greylisting" ]; then
   sleep 3
   whmapi1 disable_cpgreylist
   echo "greylisting: 第一次執行為空" >> /root/install_date
fi
if [ -z "$twofactor" ]; then
   sleep 3
   whmapi1 twofactorauth_enable_policy
   echo "twofactor: 第一次執行為空" >> /root/install_date
fi

##############################
### Install MySQL Governor ###
##############################

# # 安裝 MySQL Governor
# if [ "$install_dbgovernor" == "yes" ]; then

#    # 安裝 MySQL Governor 套件
#    dnf install governor-mysql -y

#    if [ "$mysql_version" == "8.0" ]; then
#       /usr/share/lve/dbgovernor/db-select-mysql --mysql-version=mysql80
#    fi
#    if [ "$mysql_version" == "10.5" ]; then
#       /usr/share/lve/dbgovernor/db-select-mysql --mysql-version=mariadb105
#    fi
#    if [ "$mysql_version" == "10.6" ]; then
#       /usr/share/lve/dbgovernor/db-select-mysql --mysql-version=mariadb106
#    fi
#    if [ "$mysql_version" == "10.11" ]; then
#       /usr/share/lve/dbgovernor/db-select-mysql --mysql-version=mariadb1011
#    fi
#    if [ "$mysql_version" == "11.4" ]; then
#       /usr/share/lve/dbgovernor/db-select-mysql --mysql-version=mariadb1011
#    fi

#    /usr/share/lve/dbgovernor/mysqlgovernor.py --install --yes

#    # 設定 Mysql Governor
#    dbctl --lve-mode abusers
#    dbctl set default --cpu=300,150,80,50 --read=300,120,80,30 --write=200,100,50,15 --slow=200

#    # 重啟 db_governor 服務
#    systemctl restart db_governor.service

#    # 設置開機啟動與啟動cPanel服務監控
#    systemctl enable db_governor
#    whmapi1 configureservice service='db_governor' enabled='1' monitored='1'

# fi

###################
### Install End ###
###################

# 紀錄結束安裝時間
echo " " >> /root/install_date
echo "結束安裝 : `date`" >> /root/install_date

# 說明已安裝完畢
echo ""
echo "cpinst9.sh 已執行安裝完畢，接著請依照 https://wiki.noc.tw/自用虛擬主機 初始設定進行手動設定與確認"
echo ""
echo "Zabbix 網頁頁面設定監控項目請參考 https://wiki.noc.tw/Zabbix_建立新主機"
echo ""

# Easyapache4 未正確套用指定PHP版本json檔案判斷
if [ ! -d "/etc/cpanel/ea4/profiles/custom/" ]; then

   echo "#########################"
   echo ""
   echo "注意 !!! 此次 cPanel 安裝 Easyapache4 並未正確套用指定 PHP版本 josn 檔案，請手動執行以下指令"
   echo ""
   echo "/usr/local/bin/ea_install_profile --install /etc/cpanel_initial_install_ea4_profile.json"
   echo ""
   echo "執行完畢後，進入 WHM > Easyapache4 確認目前預設 PHP版本 是否為安裝指定的版本"
   echo ""
   echo "接著執行 /opt/cpinst/cpinst/install_script/lsapi.sh 指令重新套用lsapi設定"
   echo ""
   echo "最後確認 WHM > MultiPHP Manager 功能底下 PHP Handlers 全部版本是否都改成 lsapi"
   echo ""
   echo "如果遇到不可預期的問題，請回報讓二線與主管知道這件事情，後續才能排查問題點 !!!"
   echo ""
   echo "#########################"

fi

# 確認 MySQL 服務是否正常
mysql_status=$(netstat -tulnp | grep '3306' 2>/dev/null)
if [ ! "${mysql_status}" ]; then
   echo "目前 MySQL 服務不正常，請嘗試故障排除，如果無法解決請回報二線與主管知道這件事情，後續才能排查問題點"
fi

# 程式設計人員將不定時確認cpinst部屬log，加以改善除錯，增加 cpinst 自動安裝穩定性
dnf install -y mutt
echo "cpinst 部屬完成寄送通知給程式設計人員，確認部屬過程中是否有錯誤，加以修正" | mutt -s "${MAIN_IP} 部屬 ${host_use_user} cPanel 主機環境 " -a /root/cpinst_install.log -a /root/install_date -- jacky@host.com.tw
